Type: Package
Package: performance
Title: Assessment of Regression Models Performance
Version: 0.17.0
Authors@R: 
    c(person(given = "Daniel",
             family = "Lüdecke",
             role = c("aut", "cre"),
             email = "officialeasystats@gmail.com",
             comment = c(ORCID = "0000-0002-8895-3206")),
      person(given = "Dominique",
             family = "Makowski",
             role = c("aut", "ctb"),
             email = "dom.makowski@gmail.com",
             comment = c(ORCID = "0000-0001-5375-9967")),
     person(given = "Mattan S.",
             family = "Ben-Shachar",
             role = c("aut", "ctb"),
             email = "matanshm@post.bgu.ac.il",
             comment = c(ORCID = "0000-0002-4287-4801")),
      person(given = "Indrajeet",
             family = "Patil",
             role = c("aut", "ctb"),
             email = "patilindrajeet.science@gmail.com",
             comment = c(ORCID = "0000-0003-1995-6531")),
      person(given = "Philip",
             family = "Waggoner",
             role = c("aut", "ctb"),
             email = "philip.waggoner@gmail.com",
             comment = c(ORCID = "0000-0002-7825-7573")),
      person(given = "Brenton M.",
             family = "Wiernik",
             role = c("aut", "ctb"),
             email = "brenton@wiernik.org",
             comment = c(ORCID = "0000-0001-9560-6336")),
      person(given = "Rémi",
             family = "Thériault",
             role = c("aut", "ctb"),
             email = "remi.theriault@mail.mcgill.ca",
             comment = c(ORCID = "0000-0003-4315-6788")),
      person(given = "Vincent",
             family = "Arel-Bundock",
             email = "vincent.arel-bundock@umontreal.ca",
             role = "ctb",
             comment = c(ORCID = "0000-0003-2042-7063")),
      person(given = "Martin",
             family = "Jullum",
             role = "rev"),
      person(given = "gjo11",
             role = "rev"),
      person(given = "Etienne",
             family = "Bacher",
             email = "etienne.bacher@protonmail.com",
             role = "ctb",
             comment = c(ORCID = "0000-0002-9271-5075")),
      person(given = "Joseph",
             family = "Luchman",
             role = "ctb",
             comment = c(ORCID = "0000-0002-8886-9717")))
Maintainer: Daniel Lüdecke <officialeasystats@gmail.com>
Description: Utilities for computing measures to assess model quality,
    which are not directly provided by R's 'base' or 'stats' packages.
    These include e.g. measures like r-squared, intraclass correlation
    coefficient (Nakagawa, Johnson & Schielzeth (2017)
    <doi:10.1098/rsif.2017.0213>), root mean squared error or functions to
    check models for overdispersion, singularity or zero-inflation and
    more. Functions apply to a large variety of regression models,
    including generalized linear models, mixed effects models and Bayesian
    models. References: Lüdecke et al. (2021) <doi:10.21105/joss.03139>.
License: GPL-3
URL: https://easystats.github.io/performance/
BugReports: https://github.com/easystats/performance/issues
Depends: R (>= 4.0)
Imports: bayestestR (>= 0.17.0), insight (>= 1.5.0), datawizard (>=
        1.3.1), stats, methods, utils
Suggests: AER, afex, BayesFactor, bayesplot, betareg, bigutilsr,
        blavaan, boot, brms, car, carData, CompQuadForm, correlation
        (>= 0.8.8), cplm, curl, dagitty, dbscan, DHARMa (>= 0.4.7),
        discovr, estimatr, fixest, flextable, forecast, ftExtra, gamm4,
        ggdag, glmmTMB (>= 1.1.12), GPArotation, graphics, Hmisc,
        httr2, ICS, ICSOutlier, ISLR, ivreg, lavaan, lme4, lmtest, loo,
        MASS, Matrix, mclogit, mclust, metadat, metafor, mgcv, mlogit,
        modelbased (>= 0.12.0), multimode, nestedLogit, nlme, nnet,
        nonnest2, ordinal, parallel, parameters (>= 0.28.0), patchwork,
        pscl, psych, psychTools, quantreg, qqplotr (>= 0.0.6),
        randomForest, RcppEigen, reformulas, rempsyc, rmarkdown,
        rstanarm, rstantools, sandwich, see (>= 0.13.0), survey,
        survival, testthat (>= 3.2.1), tweedie, VGAM, withr (>= 3.0.0)
Encoding: UTF-8
Language: en-US
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/Needs/website: rstudio/bslib, r-lib/pkgdown,
        easystats/easystatstemplate
Config/rcmdcheck/ignore-inconsequential-notes: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-21 20:29:27 UTC; DL
Author: Daniel Lüdecke [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8895-3206>),
  Dominique Makowski [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-5375-9967>),
  Mattan S. Ben-Shachar [aut, ctb] (ORCID:
    <https://orcid.org/0000-0002-4287-4801>),
  Indrajeet Patil [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-1995-6531>),
  Philip Waggoner [aut, ctb] (ORCID:
    <https://orcid.org/0000-0002-7825-7573>),
  Brenton M. Wiernik [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-9560-6336>),
  Rémi Thériault [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-4315-6788>),
  Vincent Arel-Bundock [ctb] (ORCID:
    <https://orcid.org/0000-0003-2042-7063>),
  Martin Jullum [rev],
  gjo11 [rev],
  Etienne Bacher [ctb] (ORCID: <https://orcid.org/0000-0002-9271-5075>),
  Joseph Luchman [ctb] (ORCID: <https://orcid.org/0000-0002-8886-9717>)
Repository: CRAN
Date/Publication: 2026-05-21 21:50:02 UTC
Built: R 4.6.0; ; 2026-05-22 00:19:31 UTC; unix
