# Generated by roxygen2: do not edit by hand


S3method(bayesplot::pp_check, lm)
S3method(bayesplot::pp_check, glm)
S3method(bayesplot::pp_check, glmmTMB)
S3method(bayesplot::pp_check, glm.nb)
S3method(bayesplot::pp_check, merMod)
S3method(bayesplot::pp_check, MixMod)
S3method(bayesplot::pp_check, mle2)
S3method(bayesplot::pp_check, negbin)
S3method(bayesplot::pp_check, polr)
S3method(bayesplot::pp_check, rma)
S3method(bayesplot::pp_check, vlm)
S3method(bayesplot::pp_check, wbm)
S3method(bayesplot::pp_check, BFBayesFactor)
S3method(AIC,bife)
S3method(as.data.frame,check_outliers)
S3method(as.data.frame,icc)
S3method(as.data.frame,looic)
S3method(as.data.frame,performance_accuracy)
S3method(as.data.frame,performance_pcp)
S3method(as.data.frame,performance_score)
S3method(as.data.frame,r2_bayes)
S3method(as.data.frame,r2_loo)
S3method(as.data.frame,r2_nakagawa)
S3method(as.double,check_outliers)
S3method(as.double,item_omega)
S3method(as.double,performance_roc)
S3method(check_autocorrelation,DHARMa)
S3method(check_autocorrelation,default)
S3method(check_autocorrelation,performance_simres)
S3method(check_collinearity,BFBayesFactor)
S3method(check_collinearity,MixMod)
S3method(check_collinearity,afex_aov)
S3method(check_collinearity,betamfx)
S3method(check_collinearity,betaor)
S3method(check_collinearity,default)
S3method(check_collinearity,glmmTMB)
S3method(check_collinearity,hurdle)
S3method(check_collinearity,logitmfx)
S3method(check_collinearity,logitor)
S3method(check_collinearity,negbinirr)
S3method(check_collinearity,negbinmfx)
S3method(check_collinearity,poissonirr)
S3method(check_collinearity,poissonmfx)
S3method(check_collinearity,probitmfx)
S3method(check_collinearity,zerocount)
S3method(check_collinearity,zeroinfl)
S3method(check_concurvity,gam)
S3method(check_distribution,default)
S3method(check_distribution,numeric)
S3method(check_group_variation,data.frame)
S3method(check_group_variation,default)
S3method(check_heteroscedasticity,default)
S3method(check_heteroscedasticity,glm)
S3method(check_heteroscedasticity,glmmTMB)
S3method(check_homogeneity,afex_aov)
S3method(check_homogeneity,default)
S3method(check_homogeneity,htest)
S3method(check_model,DHARMa)
S3method(check_model,brmsfit)
S3method(check_model,default)
S3method(check_model,model_fit)
S3method(check_model,performance_simres)
S3method(check_model,stanreg)
S3method(check_multimodal,data.frame)
S3method(check_multimodal,numeric)
S3method(check_normality,BFBayesFactor)
S3method(check_normality,afex_aov)
S3method(check_normality,default)
S3method(check_normality,fa)
S3method(check_normality,glm)
S3method(check_normality,glmmTMB)
S3method(check_normality,htest)
S3method(check_normality,item_omega)
S3method(check_normality,lmerModLmerTest)
S3method(check_normality,merMod)
S3method(check_normality,numeric)
S3method(check_normality,omega)
S3method(check_normality,parameters_efa)
S3method(check_normality,performance_simres)
S3method(check_normality,principal)
S3method(check_outliers,BFBayesFactor)
S3method(check_outliers,DHARMa)
S3method(check_outliers,character)
S3method(check_outliers,data.frame)
S3method(check_outliers,default)
S3method(check_outliers,fa)
S3method(check_outliers,fixest)
S3method(check_outliers,fixest_multi)
S3method(check_outliers,geeglm)
S3method(check_outliers,glmmTMB)
S3method(check_outliers,glmrob)
S3method(check_outliers,gls)
S3method(check_outliers,grouped_df)
S3method(check_outliers,item_omega)
S3method(check_outliers,lme)
S3method(check_outliers,lmrob)
S3method(check_outliers,meta)
S3method(check_outliers,metabin)
S3method(check_outliers,metagen)
S3method(check_outliers,numeric)
S3method(check_outliers,omega)
S3method(check_outliers,parameters_efa)
S3method(check_outliers,performance_simres)
S3method(check_outliers,psych)
S3method(check_outliers,rma)
S3method(check_outliers,rma.uni)
S3method(check_outliers,rq)
S3method(check_outliers,rqs)
S3method(check_outliers,rqss)
S3method(check_overdispersion,DHARMa)
S3method(check_overdispersion,default)
S3method(check_overdispersion,fixest)
S3method(check_overdispersion,fixest_multi)
S3method(check_overdispersion,glm)
S3method(check_overdispersion,glmmTMB)
S3method(check_overdispersion,glmx)
S3method(check_overdispersion,merMod)
S3method(check_overdispersion,model_fit)
S3method(check_overdispersion,negbin)
S3method(check_overdispersion,negbinirr)
S3method(check_overdispersion,negbinmfx)
S3method(check_overdispersion,performance_simres)
S3method(check_overdispersion,poissonirr)
S3method(check_overdispersion,poissonmfx)
S3method(check_predictions,BFBayesFactor)
S3method(check_predictions,brmsfit)
S3method(check_predictions,default)
S3method(check_predictions,lme)
S3method(check_predictions,stanreg)
S3method(check_priors,brmsfit)
S3method(check_priors,default)
S3method(check_priors,stanreg)
S3method(check_residuals,DHARMa)
S3method(check_residuals,default)
S3method(check_residuals,fa)
S3method(check_residuals,item_omega)
S3method(check_residuals,omega)
S3method(check_residuals,parameters_efa)
S3method(check_residuals,performance_simres)
S3method(check_residuals,principal)
S3method(check_singularity,MixMod)
S3method(check_singularity,clmm)
S3method(check_singularity,cpglmm)
S3method(check_singularity,default)
S3method(check_singularity,glmmTMB)
S3method(check_singularity,glmmadmb)
S3method(check_singularity,lme)
S3method(check_singularity,merMod)
S3method(check_singularity,rlmerMod)
S3method(check_sphericity,Anova.mlm)
S3method(check_sphericity,afex_aov)
S3method(check_sphericity,default)
S3method(check_sphericity,mlm)
S3method(check_symmetry,htest)
S3method(check_symmetry,numeric)
S3method(check_zeroinflation,DHARMa)
S3method(check_zeroinflation,default)
S3method(check_zeroinflation,performance_simres)
S3method(cronbachs_alpha,data.frame)
S3method(cronbachs_alpha,matrix)
S3method(cronbachs_alpha,parameters_efa)
S3method(cronbachs_alpha,parameters_pca)
S3method(display,check_itemscale)
S3method(display,compare_performance)
S3method(display,item_difficulty)
S3method(display,item_discrimination)
S3method(display,item_omega)
S3method(display,item_reliability)
S3method(display,performance_model)
S3method(display,test_likelihoodratio)
S3method(display,test_performance)
S3method(fitted,BFBayesFactor)
S3method(format,compare_performance)
S3method(format,performance_model)
S3method(format,performance_rmse)
S3method(format,test_likelihoodratio)
S3method(format,test_performance)
S3method(item_omega,data.frame)
S3method(item_omega,matrix)
S3method(logLik,cpglm)
S3method(logLik,iv_robust)
S3method(logLik,ivreg)
S3method(logLik,plm)
S3method(logLik,svycoxph)
S3method(model_performance,Arima)
S3method(model_performance,BFBayesFactor)
S3method(model_performance,DirichletRegModel)
S3method(model_performance,MixMod)
S3method(model_performance,bayesx)
S3method(model_performance,betamfx)
S3method(model_performance,betaor)
S3method(model_performance,betareg)
S3method(model_performance,bife)
S3method(model_performance,blavaan)
S3method(model_performance,brmsfit)
S3method(model_performance,censReg)
S3method(model_performance,clm)
S3method(model_performance,clm2)
S3method(model_performance,coxph)
S3method(model_performance,coxph_weightit)
S3method(model_performance,default)
S3method(model_performance,fa)
S3method(model_performance,felm)
S3method(model_performance,fixest)
S3method(model_performance,fixest_multi)
S3method(model_performance,flexsurvreg)
S3method(model_performance,glm)
S3method(model_performance,glm_weightit)
S3method(model_performance,glmmTMB)
S3method(model_performance,glmmadmb)
S3method(model_performance,glmx)
S3method(model_performance,hurdle)
S3method(model_performance,item_omega)
S3method(model_performance,iv_robust)
S3method(model_performance,ivreg)
S3method(model_performance,kmeans)
S3method(model_performance,lavaan)
S3method(model_performance,lm)
S3method(model_performance,lm_robust)
S3method(model_performance,lme)
S3method(model_performance,lmrob)
S3method(model_performance,logitmfx)
S3method(model_performance,logitor)
S3method(model_performance,margins)
S3method(model_performance,merMod)
S3method(model_performance,mixed)
S3method(model_performance,mixor)
S3method(model_performance,mlogit)
S3method(model_performance,model_fit)
S3method(model_performance,multinom)
S3method(model_performance,multinom_weightit)
S3method(model_performance,negbinirr)
S3method(model_performance,negbinmfx)
S3method(model_performance,nestedLogit)
S3method(model_performance,omega)
S3method(model_performance,ordinal_weightit)
S3method(model_performance,parameters_efa)
S3method(model_performance,plm)
S3method(model_performance,poissonirr)
S3method(model_performance,poissonmfx)
S3method(model_performance,polr)
S3method(model_performance,principal)
S3method(model_performance,probitmfx)
S3method(model_performance,rlmerMod)
S3method(model_performance,rma)
S3method(model_performance,sem)
S3method(model_performance,stanmvreg)
S3method(model_performance,stanreg)
S3method(model_performance,survreg)
S3method(model_performance,svyglm)
S3method(model_performance,truncreg)
S3method(model_performance,vglm)
S3method(model_performance,zeroinfl)
S3method(model_performance,zerotrunc)
S3method(performance_aic,bayesx)
S3method(performance_aic,betamfx)
S3method(performance_aic,betaor)
S3method(performance_aic,default)
S3method(performance_aic,lmerMod)
S3method(performance_aic,logitmfx)
S3method(performance_aic,logitor)
S3method(performance_aic,negbinirr)
S3method(performance_aic,negbinmfx)
S3method(performance_aic,poissonirr)
S3method(performance_aic,poissonmfx)
S3method(performance_aic,probitmfx)
S3method(performance_aic,svycoxph)
S3method(performance_aic,svyglm)
S3method(performance_aic,vgam)
S3method(performance_aic,vglm)
S3method(performance_aicc,Arima)
S3method(performance_aicc,bife)
S3method(performance_aicc,default)
S3method(performance_aicc,lmerMod)
S3method(performance_aicc,rma)
S3method(performance_aicc,vglm)
S3method(performance_dvour,default)
S3method(performance_dvour,estimate_grouplevel)
S3method(performance_logloss,brmsfit)
S3method(performance_logloss,default)
S3method(performance_logloss,logitmfx)
S3method(performance_logloss,logitor)
S3method(performance_logloss,probitmfx)
S3method(performance_mae,betamfx)
S3method(performance_mae,betaor)
S3method(performance_mae,default)
S3method(performance_mae,logitmfx)
S3method(performance_mae,logitor)
S3method(performance_mae,model_fit)
S3method(performance_mae,negbinirr)
S3method(performance_mae,negbinmfx)
S3method(performance_mae,poissonirr)
S3method(performance_mae,poissonmfx)
S3method(performance_mae,probitmfx)
S3method(performance_mse,betamfx)
S3method(performance_mse,betaor)
S3method(performance_mse,default)
S3method(performance_mse,logitmfx)
S3method(performance_mse,logitor)
S3method(performance_mse,model_fit)
S3method(performance_mse,negbinirr)
S3method(performance_mse,negbinmfx)
S3method(performance_mse,poissonirr)
S3method(performance_mse,poissonmfx)
S3method(performance_mse,probitmfx)
S3method(performance_reliability,default)
S3method(plot,binned_residuals)
S3method(plot,check_autocorrelation)
S3method(plot,check_clusterstructure)
S3method(plot,check_collinearity)
S3method(plot,check_dag)
S3method(plot,check_distribution)
S3method(plot,check_distribution_numeric)
S3method(plot,check_heteroscedasticity)
S3method(plot,check_homogeneity)
S3method(plot,check_model)
S3method(plot,check_normality)
S3method(plot,check_outliers)
S3method(plot,check_overdisp)
S3method(plot,check_residuals)
S3method(plot,check_sphericity)
S3method(plot,compare_performance)
S3method(plot,performance_check_priors)
S3method(plot,performance_pp_check)
S3method(plot,performance_roc)
S3method(plot,performance_simres)
S3method(plot,test_likelihoodratio)
S3method(plot,test_performance)
S3method(print,binned_residuals)
S3method(print,check_autocorrelation)
S3method(print,check_collinearity)
S3method(print,check_concurvity)
S3method(print,check_dag)
S3method(print,check_distribution)
S3method(print,check_distribution_numeric)
S3method(print,check_group_variation)
S3method(print,check_heterogeneity_bias)
S3method(print,check_heteroscedasticity)
S3method(print,check_homogeneity)
S3method(print,check_itemscale)
S3method(print,check_model)
S3method(print,check_normality)
S3method(print,check_normality_binom)
S3method(print,check_outliers)
S3method(print,check_outliers_metafor)
S3method(print,check_outliers_metagen)
S3method(print,check_outliers_simres)
S3method(print,check_overdisp)
S3method(print,check_residuals)
S3method(print,check_sphericity)
S3method(print,check_symmetry)
S3method(print,check_zi)
S3method(print,compare_performance)
S3method(print,icc)
S3method(print,icc_by_group)
S3method(print,icc_decomposed)
S3method(print,item_difficulty)
S3method(print,item_discrimination)
S3method(print,item_omega)
S3method(print,item_reliability)
S3method(print,looic)
S3method(print,performance_accuracy)
S3method(print,performance_cv)
S3method(print,performance_hosmer)
S3method(print,performance_ior)
S3method(print,performance_model)
S3method(print,performance_mor)
S3method(print,performance_omega)
S3method(print,performance_pcp)
S3method(print,performance_poor)
S3method(print,performance_pp_check)
S3method(print,performance_rmse)
S3method(print,performance_roc)
S3method(print,performance_score)
S3method(print,performance_simres)
S3method(print,r2_bayes)
S3method(print,r2_generic)
S3method(print,r2_loo)
S3method(print,r2_mlm)
S3method(print,r2_nakagawa)
S3method(print,r2_nakagawa_by_group)
S3method(print,r2_pseudo)
S3method(print,test_likelihoodratio)
S3method(print,test_performance)
S3method(print_html,check_group_variation)
S3method(print_html,check_itemscale)
S3method(print_html,compare_performance)
S3method(print_html,item_difficulty)
S3method(print_html,item_discrimination)
S3method(print_html,item_omega)
S3method(print_html,item_reliability)
S3method(print_html,test_likelihoodratio)
S3method(print_html,test_performance)
S3method(print_md,check_itemscale)
S3method(print_md,compare_performance)
S3method(print_md,item_difficulty)
S3method(print_md,item_discrimination)
S3method(print_md,item_omega)
S3method(print_md,item_reliability)
S3method(print_md,performance_model)
S3method(print_md,test_likelihoodratio)
S3method(print_md,test_performance)
S3method(r2,Arima)
S3method(r2,BBreg)
S3method(r2,BFBayesFactor)
S3method(r2,DirichletRegModel)
S3method(r2,MixMod)
S3method(r2,aov)
S3method(r2,bayesx)
S3method(r2,betamfx)
S3method(r2,betaor)
S3method(r2,betareg)
S3method(r2,bife)
S3method(r2,bigglm)
S3method(r2,biglm)
S3method(r2,bracl)
S3method(r2,brmsfit)
S3method(r2,brmultinom)
S3method(r2,censReg)
S3method(r2,clm)
S3method(r2,clm2)
S3method(r2,clmm)
S3method(r2,complmrob)
S3method(r2,coxph)
S3method(r2,cpglm)
S3method(r2,cpglmm)
S3method(r2,cph)
S3method(r2,crch)
S3method(r2,default)
S3method(r2,feis)
S3method(r2,felm)
S3method(r2,fixest)
S3method(r2,fixest_multi)
S3method(r2,gam)
S3method(r2,gamlss)
S3method(r2,glm)
S3method(r2,glmmTMB)
S3method(r2,glmmadmb)
S3method(r2,glmx)
S3method(r2,hurdle)
S3method(r2,iv_robust)
S3method(r2,ivreg)
S3method(r2,lm)
S3method(r2,lm_robust)
S3method(r2,lme)
S3method(r2,lmrob)
S3method(r2,logitmfx)
S3method(r2,logitor)
S3method(r2,lrm)
S3method(r2,mblogit)
S3method(r2,mclogit)
S3method(r2,merMod)
S3method(r2,mhurdle)
S3method(r2,mixed)
S3method(r2,mlm)
S3method(r2,mlogit)
S3method(r2,mmclogit)
S3method(r2,model_fit)
S3method(r2,multinom)
S3method(r2,negbinirr)
S3method(r2,negbinmfx)
S3method(r2,nestedLogit)
S3method(r2,ols)
S3method(r2,oohbchoice)
S3method(r2,phylolm)
S3method(r2,plm)
S3method(r2,poissonirr)
S3method(r2,poissonmfx)
S3method(r2,polr)
S3method(r2,probitmfx)
S3method(r2,rlmerMod)
S3method(r2,rma)
S3method(r2,scam)
S3method(r2,selection)
S3method(r2,sem)
S3method(r2,serp)
S3method(r2,stanreg)
S3method(r2,summary.lm)
S3method(r2,survreg)
S3method(r2,svyglm)
S3method(r2,systemfit)
S3method(r2,truncreg)
S3method(r2,vgam)
S3method(r2,vglm)
S3method(r2,wbm)
S3method(r2,zeroinfl)
S3method(r2,zerotrunc)
S3method(r2_coxsnell,BBreg)
S3method(r2_coxsnell,DirichletRegModel)
S3method(r2_coxsnell,bayesx)
S3method(r2_coxsnell,bife)
S3method(r2_coxsnell,censReg)
S3method(r2_coxsnell,clm)
S3method(r2_coxsnell,clm2)
S3method(r2_coxsnell,coxph)
S3method(r2_coxsnell,cpglm)
S3method(r2_coxsnell,crch)
S3method(r2_coxsnell,glm)
S3method(r2_coxsnell,glmmTMB)
S3method(r2_coxsnell,glmx)
S3method(r2_coxsnell,logitmfx)
S3method(r2_coxsnell,logitor)
S3method(r2_coxsnell,mblogit)
S3method(r2_coxsnell,mclogit)
S3method(r2_coxsnell,multinom)
S3method(r2_coxsnell,negbinirr)
S3method(r2_coxsnell,negbinmfx)
S3method(r2_coxsnell,nestedLogit)
S3method(r2_coxsnell,poissonirr)
S3method(r2_coxsnell,poissonmfx)
S3method(r2_coxsnell,polr)
S3method(r2_coxsnell,probit)
S3method(r2_coxsnell,serp)
S3method(r2_coxsnell,survreg)
S3method(r2_coxsnell,svycoxph)
S3method(r2_coxsnell,truncreg)
S3method(r2_efron,default)
S3method(r2_ferrari,default)
S3method(r2_kullback,default)
S3method(r2_kullback,glm)
S3method(r2_loo_posterior,BFBayesFactor)
S3method(r2_loo_posterior,brmsfit)
S3method(r2_loo_posterior,stanmvreg)
S3method(r2_loo_posterior,stanreg)
S3method(r2_mcfadden,bracl)
S3method(r2_mcfadden,brmultinom)
S3method(r2_mcfadden,censReg)
S3method(r2_mcfadden,clm)
S3method(r2_mcfadden,clm2)
S3method(r2_mcfadden,cpglm)
S3method(r2_mcfadden,glm)
S3method(r2_mcfadden,glmmTMB)
S3method(r2_mcfadden,glmx)
S3method(r2_mcfadden,logitmfx)
S3method(r2_mcfadden,logitor)
S3method(r2_mcfadden,mblogit)
S3method(r2_mcfadden,mclogit)
S3method(r2_mcfadden,mlogit)
S3method(r2_mcfadden,multinom)
S3method(r2_mcfadden,negbinirr)
S3method(r2_mcfadden,negbinmfx)
S3method(r2_mcfadden,poissonirr)
S3method(r2_mcfadden,poissonmfx)
S3method(r2_mcfadden,polr)
S3method(r2_mcfadden,probitmfx)
S3method(r2_mcfadden,serp)
S3method(r2_mcfadden,truncreg)
S3method(r2_mcfadden,vglm)
S3method(r2_mckelvey,default)
S3method(r2_mlm,mlm)
S3method(r2_nagelkerke,BBreg)
S3method(r2_nagelkerke,DirichletRegModel)
S3method(r2_nagelkerke,bife)
S3method(r2_nagelkerke,bracl)
S3method(r2_nagelkerke,brmultinom)
S3method(r2_nagelkerke,censReg)
S3method(r2_nagelkerke,clm)
S3method(r2_nagelkerke,clm2)
S3method(r2_nagelkerke,coxph)
S3method(r2_nagelkerke,cpglm)
S3method(r2_nagelkerke,crch)
S3method(r2_nagelkerke,glm)
S3method(r2_nagelkerke,glmmTMB)
S3method(r2_nagelkerke,glmx)
S3method(r2_nagelkerke,logitmfx)
S3method(r2_nagelkerke,logitor)
S3method(r2_nagelkerke,mblogit)
S3method(r2_nagelkerke,mclogit)
S3method(r2_nagelkerke,multinom)
S3method(r2_nagelkerke,negbinirr)
S3method(r2_nagelkerke,negbinmfx)
S3method(r2_nagelkerke,nestedLogit)
S3method(r2_nagelkerke,poissonirr)
S3method(r2_nagelkerke,poissonmfx)
S3method(r2_nagelkerke,polr)
S3method(r2_nagelkerke,probitmfx)
S3method(r2_nagelkerke,serp)
S3method(r2_nagelkerke,survreg)
S3method(r2_nagelkerke,svycoxph)
S3method(r2_nagelkerke,truncreg)
S3method(r2_posterior,BFBayesFactor)
S3method(r2_posterior,brmsfit)
S3method(r2_posterior,stanmvreg)
S3method(r2_posterior,stanreg)
S3method(r2_tjur,default)
S3method(r2_tjur,nestedLogit)
S3method(residuals,BFBayesFactor)
S3method(residuals,check_normality_numeric)
S3method(residuals,iv_robust)
S3method(residuals,performance_simres)
S3method(rstudent,check_normality_numeric)
S3method(summary,check_group_variation)
S3method(summary,item_omega)
S3method(test_bf,ListModels)
S3method(test_bf,default)
S3method(test_likelihoodratio,ListNestedRegressions)
S3method(test_likelihoodratio,default)
S3method(test_performance,ListNestedRegressions)
S3method(test_performance,ListNonNestedRegressions)
S3method(test_performance,default)
S3method(test_vuong,ListNestedRegressions)
S3method(test_vuong,ListNonNestedRegressions)
S3method(test_vuong,default)
S3method(test_wald,ListNestedRegressions)
S3method(test_wald,ListNonNestedRegressions)
S3method(test_wald,default)
export(as.dag)
export(binned_residuals)
export(check_autocorrelation)
export(check_clusterstructure)
export(check_collinearity)
export(check_concurvity)
export(check_convergence)
export(check_dag)
export(check_distribution)
export(check_factorstructure)
export(check_group_variation)
export(check_heterogeneity_bias)
export(check_heteroscedasticity)
export(check_heteroskedasticity)
export(check_homogeneity)
export(check_itemscale)
export(check_kmo)
export(check_model)
export(check_multimodal)
export(check_normality)
export(check_outliers)
export(check_overdispersion)
export(check_predictions)
export(check_priors)
export(check_residuals)
export(check_singularity)
export(check_sphericity)
export(check_sphericity_bartlett)
export(check_symmetry)
export(check_zeroinflation)
export(compare_performance)
export(cronbachs_alpha)
export(display)
export(icc)
export(interval_odds_ratio)
export(item_alpha)
export(item_difficulty)
export(item_discrimination)
export(item_intercor)
export(item_omega)
export(item_reliability)
export(item_split_half)
export(item_totalcor)
export(looic)
export(mae)
export(median_odds_ratio)
export(model_performance)
export(mse)
export(multicollinearity)
export(performance)
export(performance_accuracy)
export(performance_aic)
export(performance_aicc)
export(performance_cv)
export(performance_dvour)
export(performance_hosmer)
export(performance_ior)
export(performance_logloss)
export(performance_mae)
export(performance_mor)
export(performance_mse)
export(performance_pcp)
export(performance_poor)
export(performance_reliability)
export(performance_rmse)
export(performance_roc)
export(performance_rse)
export(performance_score)
export(print_html)
export(print_md)
export(r2)
export(r2_bayes)
export(r2_coxsnell)
export(r2_efron)
export(r2_ferrari)
export(r2_kullback)
export(r2_loo)
export(r2_loo_posterior)
export(r2_mcfadden)
export(r2_mckelvey)
export(r2_mlm)
export(r2_nagelkerke)
export(r2_nakagawa)
export(r2_posterior)
export(r2_somers)
export(r2_tjur)
export(r2_xu)
export(r2_zeroinflated)
export(rmse)
export(simulate_residuals)
export(test_bf)
export(test_likelihoodratio)
export(test_lrt)
export(test_performance)
export(test_vuong)
export(test_wald)
export(variance_decomposition)
importFrom(insight,
  display,
  print_html,
  print_md
)
importFrom(stats,
  residuals,
  rstudent
)
